import unittest
import sky_wiremock
from sky_wiremock.sky_wiremock import Wiremock
from sky_wiremock.utils import can_connect_to


@unittest.skipUnless(can_connect_to("localhost",8080), "Wiremock server not found")
class TestSkyWiremock(unittest.TestCase):

    def test_add_mapping(self):
        w = Wiremock( "localhost", 8080)
        id = w.add_mapping({
            "request": {
                "method": "GET",
                "url": "/some/thing"
            },
            "response": {
                "status": 200,
                "body": "Hello world!",
                "headers": {
                    "Content-Type": "text/plain"
                }
            } 
        })
        self.assertTrue(isinstance(id, str))
        self.assertTrue(id != -1)


    def test_populate(self):
        w = Wiremock( "localhost", 8080)
        ids = w.populate([{
            "request": {
                "method": "GET",
                "url": "/some/thing"
            },
            "response": {
                "status": 200,
                "body": "Hello world!",
                "headers": {
                    "Content-Type": "text/plain"
                }
            }
        },{
            "request": {
                "method": "GET",
                "url": "/some/thing/else"
            },
                "response": {
                "status": 200,
                "body": "Hello world again!",
                "headers": {
                    "Content-Type": "text/plain"
                }
            }
        }])
        # the method should return a list with all the new mapping ids
        # or -1 in case of errors
        self.assertTrue(isinstance(ids, list))
        self.assertEqual(len(ids), 2)


    def test_delete_mapping(self):
        w = Wiremock( "localhost", 8080)
        id = w.add_mapping({
            "request": {
                "method": "GET",
                "url": "/some/thing/to/delete"
            },
            "response": {
                "status": 200,
                "body": "Hello world!",
                "headers": {
                    "Content-Type": "text/plain"
                }
            } 
        })
        ret = w.delete_mapping(id)
        self.assertTrue(isinstance(ret, str))
        self.assertTrue(ret != -1)

    def test_mappings(self):
        w = Wiremock( "localhost", 8080)
        w.delete_all_mappings()
        id = w.add_mapping({
            "request": {
                "method": "GET",
                "url": "/some/thing/to/delete"
            },
            "response": {
                "status": 200,
                "body": "Hello world!",
                "headers": {
                    "Content-Type": "text/plain"
                }
            } 
        })
        mappings = w.mappings()
        self.assertTrue(isinstance(mappings, list))
        self.assertEqual(len(mappings), 1)


    def test_mapping_by_id(self):
        w = Wiremock( "localhost", 8080)
        w.delete_all_mappings()
        id = w.add_mapping({
            "request": {
                "method": "GET",
                "url": "/some/thing/to/delete"
            },
            "response": {
                "status": 200,
                "body": "Hello world!",
                "headers": {
                    "Content-Type": "text/plain"
                }
            } 
        })
        mapping = w.mapping_by_id(id)
        self.assertTrue(isinstance(mapping, dict))
        self.assertEqual(mapping["id"], id)

    def test_mapping_by_path_and_method(self):
        w = Wiremock( "localhost", 8080)
        w.delete_all_mappings()
        id = w.add_mapping({
            "request": {
                "method": "GET",
                "url": "/some/thing/to/delete"
            },
            "response": {
                "status": 200,
                "body": "Hello world!",
                "headers": {
                    "Content-Type": "text/plain"
                }
            } 
        })
        mapping = w.mapping_by_path_and_method("/some/thing/to/delete", "GET")
        self.assertTrue(isinstance(mapping, dict))
        self.assertEqual(mapping["id"], id)


    def test_fixed_delay(self):
        w = Wiremock( "localhost", 8080)
        w.delete_all_mappings()
        mapping = {
            "request": {
                "method": "GET",
                "url": "/some/thing/to/delete"
            },
            "response": {
                "status": 200,
                "body": "Hello world!",
                "headers": {
                    "Content-Type": "text/plain"
                }
            } 
        }
        id = w.add_mapping(mapping)
        delayed_id = w.fixed_delay(mapping, 10000)
        self.assertTrue(isinstance(delayed_id, str))
        self.assertEqual(delayed_id, id)


    def test_global_fixed_delay(self):
        w = Wiremock( "localhost", 8080)
        ret = w.global_fixed_delay(300)
        self.assertEqual(ret, 1)


    def test_random_delay(self):
        w = Wiremock( "localhost", 8080)
        w.delete_all_mappings()
        mapping = {
            "request": {
                "method": "GET",
                "url": "/some/thing/to/delete"
            },
            "response": {
                "status": 200,
                "body": "Hello world!",
                "headers": {
                    "Content-Type": "text/plain"
                }
            } 
        }
        id = w.add_mapping(mapping)
        delayed_id = w.random_delay(mapping, {
            "type": "lognormal",
            "median": 80,
            "sigma": 0.4
        })
        self.assertTrue(isinstance(delayed_id, str))
        self.assertEqual(delayed_id, id)


    def test_global_random_delay(self):
        w = Wiremock( "localhost", 8080)
        ret = w.global_random_delay({
            "type": "lognormal",
            "median": 90,
            "sigma": 0.1
        })
        self.assertEqual(ret, 1)


    def test_chunked_dribble_delay(self):
        w = Wiremock( "localhost", 8080)
        w.delete_all_mappings()
        mapping = {
            "request": {
                "method": "GET",
                "url": "/some/thing/to/delete"
            },
            "response": {
                "status": 200,
                "body": "Hello world!",
                "headers": {
                    "Content-Type": "text/plain"
                }
            } 
        }
        id = w.add_mapping(mapping)
        delayed_id = w.chunked_dribble_delay(mapping, {
            "numberOfChunks": 5,
            "totalDuration": 1000
        })
        self.assertTrue(isinstance(delayed_id, str))
        self.assertEqual(delayed_id, id)

        map = w.mapping_by_id(delayed_id)
        self.assertEqual(map["response"]["chunkedDribbleDelay"]["totalDuration"], 1000)
